body {
    background: #111717;
    padding: 0;
    margin: 0;
    font-family: Arial, Helvetica, sans-serif;
    font-size: 14px;
    color: #fff;
    padding-bottom: 20px;
    padding-top: 93px;
}

a {
    color: #308dbf;
    text-decoration: none;
}

.top {
    background: url(img/top_bg.png);
    width: 100%;
    height: 93px;
    color: #000;
    position: fixed;
    top: 0px;
    left: 0px;
    z-index: 1;
}
.top_content {
    width: 1000px;
    margin-left: auto;
    margin-right: auto;
    padding-top: 20px;
}
.top .logo {
    float: left;
    cursor: pointer;
}


.top .boxes {
    float:right;
}

.top_box {
    float: left;
    border-left: 1px solid #d9d9d9;
    padding-left: 20px;
    padding-right: 20px;
    height: 53px;
}
.top_box:first-child {
    border: none;
}
.top_box .label {
    color: #7a7a7a;
    font-size: 11px;
    font-weight: bold;
}
.top_box .value {
    color: #000;
    font-size: 16px;
    font-weight: bold;
}
.boxes .box_rps .value, .boxes .box_slaves .value {
    font-size: 32px;
}
.boxes .box_fail .value {
    font-size: 22px;
}
.boxes .box_status .value {
    text-transform: uppercase;
}

.boxes .box_running {
    display: none;
}
.hatching .boxes .box_running {display: block;}
.running .boxes .box_running {display: block;}
.stopped .boxes .box_running {display: block;}
.stopped .boxes .box_stop {display: none;}

.main {
    width: 1000px;
    margin-left: auto;
    margin-right: auto;
    position: relative;
    padding-top: 20px;
}

.start, .edit {
    width: 398px;
    position: absolute;
    left: 50%;
    top: 100px;
    margin-left: -169px;
}

.ramp {
    width:800px;
    margin-left: -370px;
}

.start .padder, .edit .padder {
    padding: 30px;
    padding-top: 0px;
}
.start h2, .edit h2 {
    color: #addf82;
    font-size: 26px;
    font-weight: bold;
}
.start label, .edit label {
    display: block;
    margin-bottom: 10px;
    margin-top: 20px;
    font-size: 16px;
}
.start input.val, .edit input.val {
    border: none;
    background: #fff;
    height: 52px;
    width: 328px;
    font-size: 24px;
    padding-left: 10px;
}
.start input.start_button, .edit input.start_button {
    margin-top: 20px;
    float: right;
}


.stopped .start {
    display: none;
    border-radius: 5px;
    -moz-border-radius: 5px;
    -webkit-border-radisu: 5px;
    border: 3px solid #eee;
    background: #111717;
}

.stopped .edit {display: none;}
.running .edit, .hatching .edit {
    display: none;
    border-radius: 5px;
    -moz-border-radius: 5px;
    -webkit-border-radisu: 5px;
    border: 3px solid #eee;
    background: #111717;
}
.running .start, .hatching .start {display: none;}

.ready .edit {display: none;}
.ready .start {display: block;}

.running .status, .hatching .status {display: block;}
.stopped .status {display: block;}
.ready .status {display: none;}

.stopped .boxes .edit_test, .ready .boxes .edit_test {display: none;}
.stopped .boxes .user_count, .ready .boxes .user_count {display: none;}

.running a.new_test, .ready a.new_test, .hatching a.new_test {display: none;}
.running a.new_test {display: none;}
.stopped a.new_test {display: block;}

.start a.close_link, .edit a.close_link{
    position: absolute;
    right: 10px;
    top: 10px;
}
.stopped .start a.close_link {display: inline;}
.running .start a.close_link, .ready .start a.close_link, .hatching .start a.close_link  {display: none;}

.stopped .edit a.close_link, .ready .edit a.close_link {display: none;}
.running .edit a.close_link, .hatching .edit a.close_link {display: inline;}

.ready .ramp {display: none;}

.ready .ramp_text {display: inline;}
.hatching .ramp_text, .running .ramp_text, .stopped .ramp_text {display: none;}

.stats_label {
    cursor: pointer;
}

.status table {
    border-collapse: collapse;
    width: 100%;
}
.status thead {
    background-color: #000;
    padding: 0;
    margin: 0;
    color: #6d6d6d;
    border-bottom: 1px solid #292f2f;
}
.status td, .status th {
    padding: 10px;
    margin: 0;
}
.status td.numeric {
    text-align: right;
}
.status tr.dark {
    background:#0d1111;
}
.status th {
    font-weight: normal;
}
.status th.error_count {
    width: 70px;
}

#errors td, #errors th {
    text-align: left;
}

#exceptions th.exception_occurences {
    width: 110px;
    text-align: center;
}
#exceptions th.exception_traceback {
    text-align: left;
}

#exceptions td.occurences {
    text-align: center;
}
#exceptions td.traceback {
    font-size: 12px;
    line-height: 18px;
    font-family: monospace;
    white-space: pre;
}

em {
    width: 100%;
    color: white;
    display: block;
    margin-left: 180px;
    font-size: 20px;
    font-family: Palatino, Times New Roman, serif;
}

ul.tabs {
    margin: 0;
    padding: 0;
    list-style: none;
    margin-top: 0px;
}
ul.tabs li {
    display: block;
    float: left;
    margin-right: 10px;
    font-size: 16px;
}
ul.tabs li a.current {
    color: #fff;
    /*font-weight: bold;*/
}

.panes {
    margin-top: 20px;
}

.about {
    width: 398px;
    position: absolute;
    left: 50%;
    top: 30px;
    margin-left: -169px;
    border-radius: 5px;
    -moz-border-radius: 5px;
    -webkit-border-radisu: 5px;
    border: 3px solid #eee;
    background: #111717;
}
.about a.close_link {
    position: absolute;
    right: 10px;
    top: 10px;
}
.about .padder {
    padding: 30px;
    padding-top: 0px;
}
